#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
from gi.repository import GLib, Gtk, Gio
import gi
from pydbus import SystemBus
import ctypes
from ctypes import *
gi.require_version('Gtk', '3.0')
import dbus
import dbus.service
import dbus.mainloop.glib
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
from kysdk_logsec import kylog
from kdk_accessctl import kyaccessctl
from enum import Enum

BUS_NAME = "com.kylin.kysdk.InputControl.hid"
BUS_PATH = "/com/kylin/kysdk/InputControl/hid"
BUS_IFACE = "com.kylin.kysdk.InputControl.hid"

class KdkDeviceState(Enum):
    MDM_DEV_STATE_DISBALE = 0
    MDM_DEV_STATE_READONLY = 1
    MDM_DEV_STATE_ENABLE = 2
class HidControl(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.InputControl.hid'>
            <method name='DisableHidDevices' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='EnableHidDevices' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetHidDevices'>
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        bus = dbus.SystemBus()
        bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
        dbus.service.Object.__init__(self, bus_name, BUS_PATH)

        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")                                   
    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        log = {
            "time":self.get_host_time,
            "hostname":self.get_host_name,
            "ip":self.get_host_ip,
            "name":"hid-control",
            "lv":"info",
            "message":message
        }
        json_log = json.dumps(log, ensure_ascii=False)
        print(json_log)

        try:
            self.log.log_transmit(json_log)
        except Exception:
            print("log message error")

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def DisableHidDevices(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DisableHidDevices")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DisableHidDevices")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        try:
            self.log_transmit("开启键鼠设备管控")
        except Exception:
            print("log message 开启键鼠设备管控 error")

        if self.so.kdk_device_mdm_get_hid_status() == KdkDeviceState.MDM_DEV_STATE_DISBALE.value:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DisableHidDevices")
            return 0

        rc = self.so.kdk_device_mdm_set_hid_status(c_int(KdkDeviceState.MDM_DEV_STATE_DISBALE.value))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DisableHidDevices")
        return rc

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def EnableHidDevices(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "EnableHidDevices")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "EnableHidDevices")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        try:
            self.log_transmit("关闭键鼠设备管控")
        except Exception:
            print("log message 关闭键鼠设备管控 error")

        if self.so.kdk_device_mdm_get_hid_status() == KdkDeviceState.MDM_DEV_STATE_ENABLE.value:
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "EnableHidDevices")
            return 0

        rc = self.so.kdk_device_mdm_set_hid_status(c_int(KdkDeviceState.MDM_DEV_STATE_ENABLE.value))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "EnableHidDevices")
        return rc

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def GetHidDevices(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetHidDevices")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetHidDevices")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.so.kdk_device_mdm_get_hid_status()

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetHidDevices")
        return rc


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = dbus.SystemBus()
    bus.request_name(BUS_NAME)
    HidControl()
    loop.run()
